# sb::Admin::Build
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Build;
use strict;
use sb::Build ();
use sb::Utils ();
use sb::App::Admin ();
use base qw( sb::App::Admin );
use vars qw( $VERSION );
$VERSION = '0.16';
# 0.16 [2009/02/09] modified title_name to make it clearer
# 0.15 [2009/01/31] modified _display_build_options and _update_do_build to show debug message
# 0.14 [2007/11/30] modified _update_do_build to correct the message
# 0.13 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.12 [2007/11/05] added title_name
# 0.11 [2007/09/23] added _build_files
# 0.10 [2007/09/20] ported from Serene bach 2.*
# 0.09 [2006/02/16] changed _do_rebuild to display message correctly
# 0.08 [2006/02/03] changed _get_directories to check parent dir correctly
# 0.07 [2006/02/03] changed _open_rebuild_option to enable rebuild menu for normal user
# 0.06 [2005/10/22] changed _do_rebuild to change message for rebuilding partly
# 0.05 [2005/07/22] changed _build_files to build files correctly
# 0.04 [2005/07/19] changed _check_extra
# 0.03 [2005/07/18] changed _open_rebuild_option to add extra rebuild option
# 0.02 [2005/07/16] changed _buikd_files to change the order of building files
# 0.01 [2005/07/08] changed _build_files to create a js for cookie
# 0.00 [2005/04/09] generate
# --------------------------------------
# // defines
sub TEMPLATE      (){ 'rebuild.html' };
sub JSFILE_NAME   (){ 'script/buildprogress.js' }
sub DEFAULT_TYPE  (){ 'main' };
sub REBUILD_LEVEL (){ 1 };
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	return( $self->SUPER::js_paths, $self->static_path . JSFILE_NAME );
}
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	$title = sb->string(' &gt; Building');
	$title .= sb->string(
		  ($self->{'type'} eq 'dir')     ? ' &gt; Directories'
		: ($self->{'type'} eq 'cleanup') ? ' &gt; Clean up'
		:                                  '');
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'cat'} = undef;
	$self->{'type'} = $self->cgi->value('__type');
	$self->{'type'} = DEFAULT_TYPE if ($self->{'type'} ne 'cleanup' and $self->{'type'} ne 'dir');
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->{'type'} ne DEFAULT_TYPE and !$self->has_permission('level'=>REBUILD_LEVEL))
	{
		$self->{'type'} = DEFAULT_TYPE;
		return $self->_display('message'=>sb->string('Not authorized.'));
	}
	SWITCH_TYPE: {
		$_ = $self->{'type'};
		/^main$/ && do
		{
			$msg = $self->_update_do_build();
			last SWITCH_TYPE;
		};
		/^cleanup$/ && do
		{
			$msg = $self->_update_do_cleanup();
			last SWITCH_TYPE;
		};
		/^dir$/ && do
		{
			$msg = $self->_update_manage_dir();
			last SWITCH_TYPE;
		};
	}
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	if ($self->{'type'} ne DEFAULT_TYPE and !$self->has_permission('level'=>REBUILD_LEVEL))
	{
		$self->{'type'} = DEFAULT_TYPE;
		$param{'message'} = sb->string('Not authorized.');
	}
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	SWITCH_TYPE: {
		$_ = $self->{'type'};
		/^main$/ && do
		{
			$self->_display_build_options($temp);
			last SWITCH_TYPE;
		};
		/^cleanup$|^dir$/ && do
		{
			$self->_display_dir_manager($temp);
			last SWITCH_TYPE;
		};
	} # end of SWITCH_TYPE
	$temp->num(0);
	$temp->tag('sb_rebuild_menu_' . $self->{'type'} => 'class="current"');
	$temp->block('sb_rebuild_' . $self->{'type'} => 1);
	$temp->block('sb_rebuild_typelist' => 1) if ($self->user->stat <= REBUILD_LEVEL);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_do_build
{
	my $self = shift;
	my $msg = undef;
	my $option = $self->cgi->value('rebuild_option');
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	$option = 'index' if ($option eq '' or $self->conf->value('conf_entry_archive') eq 'None');
	my $elapse = (times)[0];
	$self->_build_files($option);
	$elapse = (times)[0] - $elapse;
	$msg .= '[#' . ($option + 1) . '] ' if ($option =~ /^\d+$/);
	$msg .= sb->string('Your weblog has been rebuilt.  Confirm your site by clicking &quot;View weblog&quot;.');
	$msg .= sprintf('%.2f seconds',$elapse) if ( sb::Config->get->value('debug_level') );
	return $msg;
}
sub _update_do_cleanup
{
	my $self = shift;
	my $base = $self->conf->value('conf_dir_base');
	my @dirs = $self->_get_directories(
		'target' => 'log',
		'filter' => '.+' . $self->conf->value('basic_suffix'),
	);
	my $num = 0;
	foreach my $check ( @dirs )
	{
		my $dir = $base . $check->{'dir'};
		foreach my $file ( @{$check->{'files'}} )
		{
			$num++ if (unlink($dir . $file));
		}
	}
	return sprintf(sb->string('Deleted %d item(s).<br />'),$num);
}
sub _update_manage_dir
{
	my $self = shift;
	my $dir = $self->conf->value('conf_dir_base');
	if ($self->cgi->value('create') ne '' and $self->cgi->value('new_dir') =~ /[a-zA-Z0-9_\-]+/)
	{ # create a directory
		$dir .= $self->cgi->value('parent_dir') if ($self->cgi->value('parent_dir') ne '');
		$dir .= $self->cgi->value('new_dir');
		if (!-e $dir)
		{
			eval { umask(0) };
			mkdir($dir,$self->conf->value('basic_dir_attr'));
			return (-e $dir)
				? sb->string('Your request has been processed.')
				: sb->string('Failed to make a directory.  Please check the permission.');
		}
		else
		{
			return sb->string('File or directory already exists.');
		}
	}
	elsif ($self->cgi->value('delete') ne '' and $self->cgi->value('delete_dir') ne '')
	{ # delete a directory
		$dir .= $self->cgi->value('delete_dir');
		rmdir($dir) if (-e $dir);
		return (!-e $dir)
			? sprintf(sb->string('Deleted %d item(s).<br />'),1)
			: sb->string('Failed to delete a directory.  Please check the permission.');
	}
	return sb->string('Unknown error.');
}
# --------------------------------------
# // private functions - display
sub _display_build_options
{
	my $self = shift;
	my $temp = shift;
	my $type = $self->conf->value('conf_entry_archive');
	return if ($type eq 'None');
	my @entries = sb::Data->load('Entry','cond'=>{'stat'=>[1,2]});
	# --- for normal build
	my $num = sb::Data->matched;
	my $max = $self->conf->value('basic_buildnum');
	my $selector = '';
	my $build_sel = int($num / $max) + 1;
	$build_sel-- if ($num % $max == 0 and $num > 0);
	for (my $i=0;$i<$build_sel;$i++)
	{
		$selector .= '<option value="' . $i . '">';
		$selector .= sprintf(
			sb->string('[#%d] Archive rebuilt (Recent:%d-%d)'),
			($i + 1),
			($i * $max) + 1,
			($i + 1) * $max
		);
		$selector .= '</option>';
	}
	$temp->num(0);
	$temp->tag('sb_rebuild_options'=>$selector);
	$temp->block('sb_rebuild_option'=>1) ;
	# --- for ajax build
	if ($type eq 'Individual')
	{
		my $max = $self->conf->value('basic_build_ajax');
		my $check = int($num / $max) + 1;
		$check-- if ($num % $max == 0 and $num > 0);
		$num = $check;
	}
	elsif ($type eq 'Monthly')
	{
		my %check = ();
		for (my $i=0;$i<@entries;$i++)
		{
			my $month = sb::Time->format(
				'time'=>$entries[$i]->date,
				'form'=>'%Year%%Mon%',
				'zone'=>$entries[$i]->tz
			);
			$check{$month} = $month if ( !defined($check{$month}) );
		}
		$num = keys(%check);
	}
	$temp->num(0);
	$temp->tag('sb_rebuild_max'=>$num);
	$temp->tag('sb_rebuild_debug'=>int(sb::Config->get->value('debug_level')));
	$temp->block('sb_rebuild_extra_script' => 1 ) if ($self->conf->value('basic_use_ajax'));
}
sub _display_dir_manager
{
	my $self = shift;
	my $temp = shift;
	my @dirs = $self->_get_directories(
		'target' => ($self->{'type'} eq 'cleanup') ? 'log' : 'both',
		'filter' => ($self->{'type'} eq 'cleanup') ? '.+' . $self->conf->value('basic_suffix') : undef,
	);
	$temp->num(0);
	$temp->tag('sb_directory_tree'=>
		$self->_directory_tree(
			'dir'       => \@dirs,
			'show_file' => ($self->{'type'} eq 'cleanup') ? 1 : undef,
		)
	);
	if ($self->{'type'} eq 'dir')
	{
		my %sel = ('all'=>undef,'del'=>undef);
		foreach my $dir ( @dirs )
		{
			my $option = '<option value="' . $dir->{'dir'} . '">' . $dir->{'dir'} . '</option>' . "\n";
			$sel{'all'} .= $option;
			$sel{'del'} .= $option if ($dir->{'num'} == 0 and !$dir->{'sub'});
		}
		$temp->tag('sb_all_directory'=>$sel{'all'});
		$temp->tag('sb_deletable_directory'=>$sel{'del'});
	}
}
# --------------------------------------
# // private functions - utilities
sub _build_files
{ # rebuild files
	my $self = shift;
	my $option  = shift;
	my $max     = $self->conf->value('basic_buildnum');
	my $type    = $self->conf->value('conf_entry_archive');
	my $builder = sb::Build->new(
		'time'      => $self->{'time'},
		'user'      => $self->{'users'},
		'cat'       => $self->{'cat'},
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
		'blog'      => sb::Data->load('Weblog','id'=>0),
	);
	$builder->entryinfo();
	if ($option ne 'index')
	{
		my @entries = sb::Data->load('Entry',
			'cond'   => {'stat'=>[1,2]},
			'sort'   => 'date',
			'order'  => 1,
			'num'    => ($option =~ /^\d+$/) ? $max + 2 : -1,
			'bgn'    => ($option =~ /^\d+$/ and $option > 0) ? ($option * $max - 1) : 0,
			'detail' => 'on',
		);
		if ($type eq 'Individual')
		{
			my $bgn = ($option =~ /^\d+$/ and $option > 0) ? 1 : 0;
			my $end = @entries;
			if ($option =~ /^\d+$/) {
				$end = $max if ($option == 0);
				$end = $max + 1 if ($end > $max + 1);
			}
			for (my $i=$bgn;$i<$end;$i++) {
				my $nxt = ($i > 0)         ? $entries[$i - 1] : undef;
				my $prv = ($i < $#entries) ? $entries[$i + 1] : undef;
				$builder->build_entry($entries[$i],'prev'=>$prv,'next'=>$nxt);
			}
		}
		elsif ($type eq 'Monthly')
		{
			my @months = ();
			foreach my $entry ( @entries )
			{
				my $month = sb::Time->format(
					'time'=>$entry->date,
					'form'=>'%Year%%Mon%',
					'zone'=>$entry->tz,
				);
				push(@months,$month);
			}
			@months = sb::Utils->remove_duplicates(@months);
			foreach my $month ( @months )
			{
				$builder->build_monthly_archive( $month );
			}
		}
	}
	if ($option eq 'index' or $option eq 'all')
	{
		foreach my $cat ( values(%{$self->{'cat'}}) )
		{ # category indexes
			next if (!$cat->idx);
			next if ($cat->dir eq '');
			$builder->build_category_index($cat->id);
		}
		$builder->set_latest_entries;
		$builder->build_top_page;
		$builder->build_feedfile('all');
		$builder->build_javascript('all') if ($type eq 'Individual');
		$builder->build_css;
		$builder->build_cookie_js('force_to_create');
	}
}
sub _get_directories
{
	my $self = shift;
	my %param = (
		'target' => 'both', # 'log','img' or 'both'
		'filter' => undef,
		@_
	);
	my @dirs = ();
	my @tree = ();
	my $num  = 0;
	my $type = $param{'target'};
	my $cond = $param{'filter'};
	push(@dirs,sb::Utils->writable_dir($self->conf->value('conf_dir_log'))) if ($type eq 'both' or $type eq 'log');
	push(@dirs,sb::Utils->writable_dir($self->conf->value('conf_dir_img'))) if ($type eq 'both' or $type eq 'img');
	@dirs = sort { $a cmp $b } sb::Utils->remove_duplicates(@dirs);
	foreach my $check (@dirs)
	{
		my $dir = $self->conf->value('conf_dir_base') . $check;
		my @sep = split('/',$check);
		my $cur = {
			'id'     => $num,
			'name'   => $sep[$#sep],
			'dir'    => $check,
			'parent' => '',
			'num'    => 0,
			'files'  => [],
			'sub'    => undef,
			'root'   => $sep[0],
			'depth'  => $#sep,
		};
		$#sep--;
		$cur->{'parent'} = $sep[$#sep] if ($#sep >= 0);
		$tree[$#tree]->{'sub'} = 1 if ($#tree >= 0 and $tree[$#tree]->{'dir'} eq join('/',@sep) . '/');
		opendir(CHECKDIR, $dir);
		my @filelist = readdir(CHECKDIR);
		closedir(CHECKDIR);
		foreach my $file (@filelist)
		{
			next if (-d  $dir . $file);
			next if (!-r $dir . $file);
			next if (!-w $dir . $file);
			next if ($cond and $file !~ /$cond/);
			push(@{$cur->{'files'}},$file);
			$cur->{'num'}++;
		}
		$num++;
		push(@tree,$cur);
	}
	return @tree;
}
sub _directory_tree
{
	my $self  = shift;
	my %param = (
		'dir'       => [],
		'branch'    => undef,   
		'class'     => -1,
		'show_file' => undef,
		'depth'     => 0,
		'root'      => undef,
		@_
	);
	my $list = '';
	my $num = $param{'class'};
	foreach my $dir ( @{$param{'dir'}} )
	{
		next if (!defined($param{'branch'}) and $dir->{'parent'} ne '');
		if ( defined($param{'branch'}) )
		{ # for sub directories
			next if ($dir->{'parent'} ne $param{'branch'});     # check parent directory
			next if ($dir->{'depth'} != ($param{'depth'} + 1)); # check depth of directory
			next if ($dir->{'root'} ne $param{'root'});         # check root directory
		}
		$num++;
		$list .= ($num % 2) ? '<li class="odd">' : '<li class="even">';
		$list .= '<span class="dir">';
		if ($param{'show_file'} and @{$dir->{'files'}})
		{
			$list .= sprintf('<a href="#" onclick="return Spica.Utils.toggleVisible(\'%s\');">%s</a>'
				,'dir' . $dir->{'id'},
				sb->string('[Details...]')
			);
		}
		$list .= '(' . int($dir->{'num'}) . ')</span>';
		$list .= $dir->{'name'};
		if ($param{'show_file'} and @{$dir->{'files'}})
		{
			$list .= "\n" . '<ol id="dir' . $dir->{'id'} . '" style="display:none;">';
			map { $list .= '<li>' . $_ . '</li>' . "\n" } @{$dir->{'files'}};
			$list .= '</ol>';
		}
		if ($dir->{'sub'})
		{
			$list .= "\n" . '<ul>';
			$list .= $self->_directory_tree(
				'dir'       => $param{'dir'},
				'branch'    => $dir->{'name'},
				'class'     => $num,
				'show_file' => $param{'show_file'},
				'depth'     => $dir->{'depth'},
				'root'      => $dir->{'root'},
			);
			$list .= '</ul>';
		}
		$list .= '</li>' . "\n";
	}
	return $list;
}
1;
__END__
