# sb::Content
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Content;
use strict;
use sb::Language ();
use sb::Config ();
use sb::Template ();
use sb::Data ();
use sb::Time ();
use sb::Text ();
use sb::Utils ();
use sb::Template ();
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.31';
# 0.31 [2009/02/08] modified initialize_callback to use sb->get->plugin instead of sb::Plugin->get
# 0.30 [2007/08/19] added deprecated functions
# 0.29 [2007/08/12] modified initialize_callback
# 0.28 [2007/08/12] added parse/name_of_callbacks to handle extend common parts
# 0.27 [2007/08/09] enabled sb::Plugin
# 0.26 [2007/08/07] changed _entry_info to make it more efficient
# 0.25 [2007/08/01] changed output to set default block in main-loop
# 0.24 [2007/08/01] got rid of "registered" from the member variables
# 0.23 [2007/07/29] modified for Serene Bach 3.*
# 0.22 [2007/04/14] changed _area_category to enable category_area for any page in category
# 0.21 [2007/04/11] changed _check_page to set page correctly
# 0.20 [2007/03/05] changed _trackback_auto_discovery
# 0.19 [2007/02/17] changed _check_page to fix a bug
# 0.18 [2007/02/13] changed _check_page
# 0.17 [2007/02/09] added {mode_name} and {mode_id}
# 0.16 [2007/02/07] added sb::Content::Category
# 0.15 [2006/10/11] added {amazon_htmlcomment}
# 0.14 [2006/10/05] changed _check_page to output a page correctly in searching
# 0.13 [2006/02/01] changed _list_comment, _list_trackback to filter objects correctly
# 0.12 [2005/10/18] changed output to receive TemplateManager object directly
# 0.11 [2005/10/15] chnaged _page to handle category page for mobile correctly
# 0.10 [2005/10/06] changed _category_tree to handle description of category.
# 0.09 [2005/08/24] changed _area_comment and _area_trackback to load data faster
# 0.08 [2005/08/03] changed _extract_entry to sort entries correctly
# 0.07 [2005/08/02] changed output, _extract_entry and _page to add 'num' option
# 0.06 [2005/08/02] changed _list_link to display correctly with empty group
# 0.05 [2005/07/27] changed _category_tree to add an option, 'no_num'
# 0.04 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.03 [2005/07/09] changed _calendar to display horizontal and vertical calendar correctly
# 0.02 [2005/07/08] changed _sequel in Entry and _entry_info
# 0.01 [2005/06/29] changed _list_link to display link group.
# 0.00 [2005/02/06] generated
# --------------------------------------
# // defines / static variables
sub ERROR_REPORT (){ "area : %s\nfield : %s\nerror : %s==========\n" };
# --------------------------------------
# // initializer
BEGIN
{
	require sb::Content::Common;
	require sb::Content::List;
	require sb::Content::Entry;
	require sb::Content::Message;
	require sb::Content::Trackback;
	require sb::Content::Profile;
	require sb::Content::Category;
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'main'      => undef,
		'entry'     => undef,
		'message'   => undef,
		'trackback' => undef,
		'profile'   => undef,
		@_
	);
	$self->SUPER::initialize(
		'error'      => '',
		'debug'      => undef,
		'main'       => {},
		'entry'      => {},
		'comment'    => {},
		'trackback'  => {},
		'profile'    => {},
	);
	$self->generate_accessor('error','debug');
	return $self->initialize_callback(%param);
}
# --------------------------------------
# // public functions
sub initialize_callback
{
	my $self = shift;
	my %extension = @_;
	# default callbacks
	$self->{'main'}      = {sb::Content::Common::_init(),sb::Content::List::_init()};
	$self->{'entry'}     = {sb::Content::Entry::_init()};
	$self->{'comment'}   = {sb::Content::Message::_init()};
	$self->{'trackback'} = {sb::Content::Trackback::_init()};
	$self->{'profile'}   = {sb::Content::Profile::_init()};
	$self->{'category'}  = {sb::Content::Category::_init()};
	# load content callbacks from plugin
	sb->get->plugin->load_content_module($self);
	# register extra callbacks
	foreach my $area ( keys( %extension ) )
	{
		next if ( !defined($extension{$area}) );
		next if ( ref($extension{$area}) ne 'HASH' );
		my %register = %{$extension{$area}};
		foreach my $callback ( keys( %register ) )
		{
			next if (ref($register{$callback}) ne 'CODE');
			$self->{$area}{$callback} = $register{$callback};
		}
	}
	# register standard area block (these blocks cannot be overridden)
	foreach ('entry','comment_area','trackback_area','profile_area','category_area')
	{
		$self->{'main'}->{$_} = 1;
	}
	return $self;
}
sub parse
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	my ($common,$blocks) = $self->name_of_callbacks('main');
	# --- block independent parts
	$cms->num(0);
	foreach my $label ( @{$common} )
	{
		eval { &{$self->{'main'}{$label}}($cms,%var); };
		$self->{'error'} .= sprintf(ERROR_REPORT,'main',$label,$@) if ($@ and $self->debug());
	}
	# --- block dependent parts
	foreach my $block ( @{$blocks} )
	{
		next if (!$cms->existed($block));
		my $num = 
			  ($block eq 'entry')          ? $self->_parse_entry($cms,%var)
			: ($block eq 'comment_area')   ? $self->_parse_comment($cms,%var)
			: ($block eq 'trackback_area') ? $self->_parse_trackback($cms,%var)
			: ($block eq 'profile_area')   ? $self->_parse_profile($cms,%var)
			: ($block eq 'category_area')  ? $self->_parse_category($cms,%var)
			:                                eval{ &{$self->{'main'}{$block}}($cms,%var) };
		$self->{'error'} .= sprintf(ERROR_REPORT,'main',$block,$@) if ($@ and $self->debug());
		$cms->block($block=>int($num));
	}
	return;
}
sub name_of_callbacks
{
	my $self = shift;
	my $target = shift;
	my (@common,@area);
	foreach my $label ( keys %{$self->{$target}} )
	{
		($label =~ /^_/) 
			? push(@common,$label)
			: push(@area,$label);
	}
	@common = sort {$a <=> $b} @common;
	@area = sort {$a <=> $b} @area;
	return( \@common,\@area );
}
# --------------------------------------
# // private functions
sub _parse_area
{
	my $self = shift;
	my ($cms,$area,$var,$data) = @_;
	my ($common,$fields) = $self->name_of_callbacks($area);
	# --- object independent parts in the block
	$cms->num(0);
	foreach my $label ( @{$common} )
	{
		eval{ &{$self->{$area}{$label}}($cms,%{$var}) };
		$self->{'error'} .= sprintf(ERROR_REPORT,$area,$label,$@) if ($@ and $self->debug);
	}
	# --- object dependent parts in the block
	my $num = 0;
	foreach my $obj ( @{$data} )
	{ 
		$cms->num($num);
		foreach my $field ( @{$fields} )
		{
			eval{ &{$self->{$area}{$field}}($cms,$obj,%{$var}) };
			$self->{'error'} .= sprintf(ERROR_REPORT,$area,$field,$@) if ($@ and $self->debug);
		}
		$num++;
	}
	return $num;
}
sub _parse_entry
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	return 0 if ( $var{'mode'} eq 'user' );
	return $self->_parse_area($cms,'entry',\%var,$var{'entry'});
}
sub _parse_comment
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	return 0 if ( $var{'mode'} ne 'ent' );
	return 0 if ( $var{'entry'}[0]->acm == 0 );
	my @list = sb::Data->load('Message',
		'sort'   => 'date',
		'cond'   => {'stat'=>1,'eid'=>$var{'id'}},
		'order'  => $var{'conf'}->value('conf_com_sort'),
	);
	$cms->block('comment'=>$self->_parse_area($cms,'comment',\%var,\@list));
	return 1;
}
sub _parse_trackback
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	return 0 if ( $var{'mode'} ne 'ent' );
	return 0 if ( $var{'entry'}[0]->atb == 0 );
	my @list = sb::Data->load('Trackback',
		'sort'   => 'date',
		'cond'   => {'stat'=>1,'eid'=>$var{'id'}},
		'order'  => $var{'conf'}->value('conf_tb_sort'),
	);
	$cms->block('trackback'=>$self->_parse_area($cms,'trackback',\%var,\@list));
	return 1;
}
sub _parse_profile
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	return 0 if ( $var{'mode'} ne 'user' );
	my $user = sb::Data->load('User','id'=>$var{'id'});
	$self->_parse_area($cms,'profile',\%var,[$user]);
	return 1;
}
sub _parse_category
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	return 0 if ( $var{'mode'} ne 'cat' );
	$self->_parse_area($cms,'category',\%var,[$var{'cat'}->{$var{'id'}}]);
	return 1;
}
# --------------------------------------
# // public functions - class method
sub output
{
	my $class = shift;
	my $cms   = shift; # (required)sb::Template object
	my %var = (
		'mode'      => undef, # [SEL.]mode => 'ent','cat','arc','srch','page','user','mob' or 'css'
		'css'       => '',    # [URI.]css path
		'page'      => 0,     # [NUM.]current page number
		'id'        => undef, # [NUM.]id (eid, cid or pid)
		'cond'      => undef, # [CHAR]condition
		'time'      => 0,     # [NUM.]current time
		'blog'      => undef, # [OBJ.]weblog data object
		'user'      => undef, # [HASH]user data
		'cat'       => undef, # [HASH]category data
		'sortedcat' => undef, # [ARRY]sorted category data
		'entryinfo' => undef, # [HASH]article information
		'entry'     => undef, # [ARRY]article data
		'entry_num' => undef, # [NUM.]number of extracted articles
		'num'       => undef, # [NUM.]number to display per page
		'extend'    => {},    # [HASH]extra callbacks
		'conf'      => sb::Config->get,   # [OBJ.]the config object
		'lang'      => sb::Language->get, # [OBJ.]the language object
		@_
	);
	return if (!$cms); # $cms must be indicated.
	return sb::Content::Common::_css_output($cms,%var) if ($var{'mode'} eq 'css'); # output css
	# --- initialize variables ---
	my $self = ref($class) ? $class : $class->new(%{$var{'extend'}});
	$var{'content'} = $self;
	$var{'num'}  = $var{'conf'}->value('conf_entry_disp') if (!$var{'num'});
	$var{'page'} = _check_page(%var);
	$var{'blog'} = sb::Data->load('Weblog','id'=>0) if (!$var{'blog'});
	$var{'user'} = {sb::Data->load_as_hash('User')} if (!$var{'user'});
	if (!$var{'cat'})
	{
		$var{'cat'} = {sb::Data->load_as_hash('Category')};
		$var{'sortedcat'} = [ sort { $b->order <=> $a->order } values(%{$var{'cat'}}) ];
	}
	if (!$var{'entry'})
	{
		$var{'entry'} = [_extract_entry(%var)];
		$var{'entry_num'} = sb::Data->matched;
	}
	$var{'entryinfo'} = _entry_info(%var) if (!$var{'entryinfo'});
	# --- parse template ---
	$self->parse($cms,%var);
	return $cms->output;
}
# --------------------------------------
# // private functions - initialization
sub _extract_entry
{
	my %var = @_;
	return sb::Data->load('Entry','id'=>$var{'id'}) if ($var{'mode'} eq 'ent'); # get specific entry
	# get entries depends on the condition
	my $bgn = ($var{'page'} == -1) ? 0 : $var{'page'} * $var{'num'};
	my $num = ($var{'page'} == -1) ? -1 : $var{'num'};
	my %cond = ($var{'mode'} eq 'page') ? ('stat' => 1) : ('stat' => [1,2]);
	my $sortor = $var{'conf'}->value('conf_entry_sort');
	if ($var{'mode'} eq 'arc')
	{
		$cond{'date'} = sb::Utils->create_date_condition($var{'cond'});
		$cond{'__range'} = { 'date' => 'tz' };
		$sortor = $var{'conf'}->value('conf_archive_sort');
	}
	elsif ($var{'mode'} eq 'cat')
	{
		$cond{'cat'} = $var{'id'};
		$cond{'__combo'} = { 'cat' => 'add' , 'add' => ',' . $var{'id'} . ',' };
		$sortor = $var{'conf'}->value('conf_archive_sort');
	}
	elsif ($var{'mode'} eq 'srch')
	{
		my $word = $var{'cond'};
		$cond{'subj'} = '/' . $word . '/';
		$cond{'body'} = '/' . $word . '/';
		$cond{'more'} = '/' . $word . '/';
		$sortor = $var{'conf'}->value('conf_archive_sort');
	}
	return sb::Data->load('Entry',
		'bgn'    => $bgn,
		'num'    => $num,
		'sort'   => 'date',
		'order'  => $sortor,
		'cond'   => \%cond,
	);
}
sub _check_page
{
	my %var = @_;
	my $page = $var{'page'};
	my $check = $var{'conf'}->value('conf_page_disp');
	$check = 0 if (  $var{'mode'} eq 'arc' 
	             and $var{'cond'} =~ /^\d{6}$/ 
	             and $var{'conf'}->value('conf_entry_archive') eq 'Monthly');
	$check = $var{'conf'}->value('conf_search_disp') if ($var{'mode'} eq 'srch');
	$check = 1 if ($var{'mode'} eq 'mob');
	$page = -1 if ($var{'mode'} ne 'page' and !$check);
	return $page;
}
sub _entry_info
{
	my %var = @_;
	my %info = (
		'latest'   => [],
		'monthly'  => {},
		'daily'    => {},
		'neighbor' => {'prev'=>undef,'next'=>undef,},
		'category' => {},
	);
	my $check_month = ($var{'mode'} eq 'arc') 
		? substr($var{'cond'},0,6) 
		: sb::Time->format(
			'time'=>$var{'time'},
			'form'=>'%Year%%Mon%',
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
	my $check = -1;
	my @array = sb::Data->load('Entry',
		'cond'  => {'stat'=>[1,2]},
		'sort'  => 'date',
		'order' => 1,
		'param' => 'add'
	);
	for (my $i=0;$i<@array;$i++)
	{
		my $entry = $array[$i];
		if ( ($var{'mode'} eq 'ent' and $var{'id'} == $entry->id)
		  or ($var{'mode'} eq 'mob' and $var{'id'} ne '' and $var{'id'} == $entry->id) )
		{
			$check = $i;
		}
		next if ($var{'fast_mode'} eq 'skip_archive_count');
		my $date = sb::Time->format(
			'time'=>$entry->date,
			'form'=>'%Year%%Mon%%Day%',
			'zone'=>$entry->tz,
		);
		my $month = substr($date,0,6);
		if ( !defined($info{'monthly'}{$month}) )
		{
			$info{'monthly'}{$month} = {
				'count' => 1,
				'name'  => sb::Time->format(
					'time'=>$entry->date,
					'form'=>$var{'conf'}->value('conf_archivelist'),
					'lang'=>$var{'conf'}->value('conf_time_lang'),
					'zone'=>$entry->tz,
				),
			};
		}
		else
		{
			$info{'monthly'}{$month}{'count'}++;
		}
		my $category = ($entry->cat == -1) ? '' : $entry->cat;
		$info{'category'}{$category}++ if ($category ne '');
		if ($entry->add ne '')
		{
			foreach ( split(',',$entry->add) )
			{
				next if ($_ eq '');
				$info{'category'}{$_}++;
			}
		}
		$info{'daily'}{$date}++ if ($month eq $check_month);
		push(@{$info{'latest'}},$entry) if ($i < $var{'conf'}->value('conf_newent_disp'));
	}
	$info{'neighbor'}{'next'} = $array[$check - 1] if ($check > 0);
	$info{'neighbor'}{'prev'} = $array[$check + 1] if ($check > -1 and $check < $#array);
	return \%info;
}
# --------------------------------------
# // deprecated functions
sub _css_output
{
	sb::Content::Common::_css_output(@_);
}
sub _common_parts
{
	sb::Content::Common::_main(@_);
}
sub _title
{
	sb::Content::Common::_title(@_);
}
sub _page
{
	sb::Content::Common::_page(@_);
}
sub _option
{
	sb::Content::Common::_option(@_);
}
sub _list_archives
{
	sb::Content::List::_archives(@_);
}
sub _list_category
{
	sb::Content::List::_category(@_);
}
sub _list_comment
{
	sb::Content::List::_comment(@_);
}
sub _list_trackback
{
	sb::Content::List::_trackback(@_);
}
sub _list_latest
{
	sb::Content::List::_latest(@_);
}
sub _list_selected
{
	sb::Content::List::_selected(@_);
}
sub _list_link
{
	sb::Content::List::_link(@_);
}
sub _list_profile
{
	sb::Content::List::_profile(@_);
}
sub _calendar
{
	sb::Content::List::_calendar(@_);
}
sub _amazon
{
	sb::Content::List::_amazon(@_);
}
sub _monthly_link
{
	sb::Content::List::_monthly_link(@_);
}
sub _category_tree
{
	sb::Content::List::_category_tree(@_);
}
sub _pad0
{
	sb::Content::List::_pad0(@_);
}
sub _register_callback
{
	die("_register_callback is deprecated.\n");
}
sub _area_entry
{
	die("_area_entry is deprecated.\n");
}
sub _area_comment
{
	die("_area_comment is deprecated.\n");
}
sub _area_trackback
{
	die("_area_trackback is deprecated.\n");
}
sub _area_profile
{
	die("_area_profile is deprecated.\n");
}
sub _area_category
{
	die("_area_category is deprecated.\n");
}
1;
__END__
